#!/usr/bin/python3

from __future__ import annotations
from requests import get
from json import load
from typing import Dict, Any
from .model.url import RequestURL
from .model.data import Data
from .model.pollutant import Pollutant
from .model.station import Station

def _isStationAlreadyExisiting(name: str, data: Data) -> boolean:
    return data.get(name) != None

def _makeStationObj(data: Dict[str, str]) -> Station:
    return Station(data['station'], data['city'], data['state'], data['country'], {})

def _makePollutantObj(data: Dict[str, str], timeStamp: int) -> Pollutant:
    return Pollutant(data['pollutant_id'], data['pollutant_min'], data['pollutant_max'], data['pollutant_avg'], data['pollutant_unit'], timeStamp, data['station'])

def _parse(content: Dict[str, Any], data: Data):
    _timeStamp = content['updated']
    for v in content['records']:
        pollutantObj = _makePollutantObj(v, _timeStamp)
        if _isStationAlreadyExisiting(v['station'], data):
            data.updateStationRecord(pollutantObj)
        else:
            data.push(_makeStationObj(v)).updateStationRecord(pollutantObj)

def _fetch(url: str) -> Dict[str, Any]:
    resp = get(url)
    content = resp.json() if resp.ok else None
    resp.close()
    return content

def request(_data: Data) -> Data:
    try:
        _req = RequestURL()
        for i in _req:
            _tmp = _fetch(i)
            if _tmp:
                _parse(_tmp, _data)
        return _data
    except Exception:
        return _data

def parseExisitingData(source: str) -> Data:
    try:
        with open(source, 'r') as fd:
            return Data.fromJSON(load(fd))
    except Exception:
        return Data([])

if __name__ == '__main__':
    print('[!]This module is designed to be used as a backend handler')
    exit(0)



#!/usr/bin/python3

from __future__ import annotations
from json import load
from os.path import dirname, abspath, join
from requests import get

class RequestURL:
    def __init__(self, config: str = abspath(join(dirname(__file__), '../config.json')), total: int = 10):
        self._config = config
        self._total = total
        with open(self._config) as fd:
            _data = load(fd)
            self._base = _data['url']
            self._api_key = _data['api-key']
            self._format = _data['format']
            self._offset = str(_data['offset'])
            self._limit = str(_data['limit'])
        self._setTotal()

    @property
    def _getTotal(self) -> int:
        return get(self._url).json().get('total')

    def _setTotal(self):
        self._total = self._getTotal

    @property
    def _url(self) -> str:
        return self._base + '?' + 'api-key=' + self._api_key + '&format=' + self._format + '&offset=' + self._offset + '&limit=' + self._limit

    def _updateOffset(self, offset: int) -> RequestURL:
        self._offset = str(offset)
        return self

    def __iter__(self):
        while int(self._offset) < self._total:
            yield self._url
            _tmp = int(self._offset) + 10
            self._updateOffset(0 if _tmp >= self._total else _tmp)
            if int(self._offset) == 0:
                break



if __name__ == "__main__":
    print('[!]This module is designed to be used as a backend handler')
    exit(0)



#!/usr/bin/python3

from __future__ import annotations
from os import mkdir
from os.path import dirname, abspath, exists
from .util import request, parseExisitingData
from json import dump
from sys import argv
from subprocess import run

def _writeToJSON(data: Dict[str, Any], sink: str):
    with open(sink, 'w') as fd:
        dump(data, fd, indent=4)

def _makeSinkDir(sink: str):
    if not exists(sink):
        mkdir(sink)

def _handleCMDInputs():
    return argv[1] if len(argv) == 2 and argv[1].endswith('.json') else None

def collect(sinkFile: str, maxLimit: int):
    sinkFile = abspath(sinkFile)
    _makeSinkDir(dirname(sinkFile))
    data = request(parseExisitingData(sinkFile))
    if data:
        _writeToJSON(data.toJSON(maxLimit), sinkFile)
        return True
    else:
        return False

def _usage():
    print('\t$ airQ `sink-file-path_( *.json )_`\n\n \x1b[3;31;50mFor making modifications on airQ-collected data ( collected prior to this run ),\n pass that JSON path, while invoking airQ ;)\x1b[0m\n')

def _displayBanner():
    run('clear')
    print('\x1b[1;6;35;50mairQ - Air Quality Data Collector\x1b[0m\n')

def main(maxLimit: int = 24*3600):
    try:
        _displayBanner()
        sink = _handleCMDInputs()
        if not sink:
            _usage()
            print('Bad Input')
        else:
            print('Working ...')
            print('\x1b[1;37;42mSuccess\x1b[0m' if collect(sink, maxLimit) else '\x1b[1;37;41mFailed\x1b[0m')
    except KeyboardInterrupt:
        print('\n\x1b[1;37;41mTerminated\x1b[0m')
    finally:
        return

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['airQ']

package_data = \
{'': ['*'], 'airQ': ['model/*']}

extras_require = \
{'dev': ['flit']}

entry_points = \
{'console_scripts': ['airQ = airQ.app:main']}

setup(name='airQ',
      version='0.2.0',
      description='airQ module initialization point',
      author='Anjan Roy',
      author_email='anjanroy@yandex.com',
      url='https://github.com/itzmeanjan/airQ',
      packages=packages,
      package_data=package_data,
      extras_require=extras_require,
      entry_points=entry_points,
      python_requires='>=3.7',
     )



#!/usr/bin/python3

from __future__ import annotations
from typing import Dict, List, Any
from .pollutant import Pollutant

class Station:
    def __init__(self, name: str, city: str, state: str, country: str, records: Dict[int, List[Pollutant]]):
        self.name = name
        self.city = city
        self.state = state
        self.country = country
        self.records = records

    def _checkRedundancy(self, pollutantId: str, container: List[Pollutant]) -> bool:
        return False if len([i for i in container if i._id == pollutantId]) == 0 \
            else True

    def push(self, record: Pollutant) -> Station:
        _holder = self.records.get(record._timestamp)
        if not _holder:
            _holder = [record]
            self.records[record._timestamp] = _holder
        else:
            if not self._checkRedundancy(record._id, _holder):
                _holder.append(record)
        return self

    def toJSON(self) -> Dict[str, Any]:
        return {'name': self.name, 'city': self.city, 'state': self.state, 'country': self.country, 'records': dict([(k, [i.toJSON() for i in v]) for k, v in self.records.items()])}

    @staticmethod
    def fromJSON(data: Dict[str, Any]) -> Station:
        return Station(data['name'], data['city'], data['state'], data['country'], dict([(int(k), [Pollutant.fromJSON(i, int(k), data['name']) for i in v]) for k, v in data['records'].items()]))

    def removeOutOfRangeValues(self, _range: int):
        for i in [i for i in self.records.keys() if i < _range]:
            self.records.pop(i)
        return self


if __name__ == '__main__':
    print('[!]This module is designed to be used as a backend handler')
    exit(0)



#!/usr/bin/python3

from __future__ import annotations
from typing import List, Dict, Any
from .station import Station
from .pollutant import Pollutant

class Data:
    def __init__(self, stations: List[Station]):
        self._stations = stations

    def __push__(self, name: str, low: int, high: int) -> int:
        if low > high:
            return 0
        elif low == high:
            return low if self._stations[low].name > name else (low + 1)
        else:
            mid = low + (high - low) // 2
            return self.__push__(name, low, mid) \
                if self._stations[mid].name > name \
                else self.__push__(name, mid + 1, high)

    def push(self, record: Station) -> Data:
        self._stations.insert(self.__push__(record.name, 0, len(self._stations) - 1), record)
        return self

    def __get__(self, name: str, low: int, high: int) -> int:
        if low > high:
            return -1
        elif low == high:
            return low if self._stations[low].name == name else -1
        else:
            mid = low + (high - low) // 2
            return self.__get__(name, low, mid) \
                if self._stations[mid].name >= name \
                else self.__get__(name, mid + 1, high)

    def get(self, name: str) -> Station:
        _tmp = self.__get__(name, 0, len(self._stations) - 1)
        return None if _tmp == -1 else self._stations[_tmp]

    def updateStationRecord(self, record: Pollutant):
        self.get(record.forStation).push(record)
        return self

    def toJSON(self, _range: int) -> Dict[str, Any]:
        return {'stations' : [i.toJSON() for i in self._removeOutOfRangeValues(_range)._stations]}

    @staticmethod
    def fromJSON(data: Dict[str, Any]) -> Data:
        return Data([Station.fromJSON(i) for i in data['stations']])

    @property
    def _getMaxTimeStamp(self) -> int:
        return max([i for i in self._stations[0].records.keys()])

    def _removeOutOfRangeValues(self, _range: int):
        _tmp = self._getMaxTimeStamp - _range
        for i in self._stations:
            i.removeOutOfRangeValues(_tmp)
        return self


if __name__ == '__main__':
    print('[!]This module is designed to be used as a backend handler')
    exit(0)
